@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultMuleConfiguration.java 19500 2010-09-09 17:29:17Z dzapata $
+ * $Id: DefaultMuleConfiguration.java 20570 2010-12-09 20:04:42Z aperepel $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -23,16 +23,16 @@ import org.mule.util.NumberUtils;
 import org.mule.util.StringUtils;
 import org.mule.util.UUID;
 
+import org.apache.commons.lang.BooleanUtils;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
 import java.net.InetAddress;
 import java.net.UnknownHostException;
 import java.nio.charset.Charset;
 
 import javax.xml.parsers.SAXParserFactory;
 
-import org.apache.commons.lang.BooleanUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-
 /**
  * Configuration info. which can be set when creating the MuleContext but becomes
  * immutable after starting the MuleContext.
@@ -42,7 +42,7 @@ public class DefaultMuleConfiguration implements MuleConfiguration, MuleContextA
 
     public static final String[] DEFAULT_STACKTRACE_FILTER = (
             "org.mule.processor.AbstractInterceptingMessageProcessor," +
-            "org.mule.processor.builder.InterceptingChainMessageProcessorBuilder")
+            "org.mule.processor.chain")
             .split(",");
 
     /**
@@ -101,7 +101,7 @@ public class DefaultMuleConfiguration implements MuleConfiguration, MuleContextA
     /**
      * Where Mule stores any runtime files to disk. Note that in container
      * mode each app will have its working dir set one level under this dir
-     * (with app's name).
+     * (with app's name) in the {@link #setMuleContext} callback.
      *
      */
     private String workingDirectory = "./.mule";
@@ -196,8 +196,16 @@ public class DefaultMuleConfiguration implements MuleConfiguration, MuleContextA
         this.muleContext = context;
         if (containerMode)
         {
+            final String muleHome = System.getProperty("mule.home");
             // in container mode the id is the app name, have each app isolate its work dir
-            this.workingDirectory = String.format("%s/%s", getWorkingDirectory(), getId());
+            if (StringUtils.isBlank(muleHome)) {
+                // fallback to current dir as a parent
+                this.workingDirectory = String.format("%s/%s", getWorkingDirectory(), getId());
+            }
+            else
+            {
+                this.workingDirectory = String.format("%s/%s/%s", muleHome.trim(), getWorkingDirectory(), getId());
+            }
         }
     }
 
