@@ -1,5 +1,5 @@
 /*
- * $Id: CoreMessages.java 19865 2010-10-08 11:04:40Z dfeist $
+ * $Id: CoreMessages.java 20267 2010-11-19 03:15:43Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -1033,7 +1033,7 @@ public class CoreMessages extends MessageFactory
     {
         return factory.createMessage(BUNDLE_PATH, 251, key);
     }
-
+    
     public static Message objectAlreadyExists(String key)
     {
         return factory.createMessage(BUNDLE_PATH, 252, key);
@@ -1321,5 +1321,16 @@ public class CoreMessages extends MessageFactory
     {
         return factory.createMessage(BUNDLE_PATH, 328);
     }
+    
+    public static Message methodWithNumParamsNotFoundOnObject(String method, int numArgments, Object object)
+    {
+        return factory.createMessage(BUNDLE_PATH, 329, method, numArgments,
+            StringMessageUtils.toString(object));
+    }
+    
+    public static Message expressionEnricherNotRegistered(String key)
+    {
+        return factory.createMessage(BUNDLE_PATH, 330, key);
+    }
 
 }
