@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultInboundEndpoint.java 19549 2010-09-10 12:40:29Z dfeist $
+ * $Id: DefaultInboundEndpoint.java 20721 2010-12-14 19:39:43Z tcarlson $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -30,6 +30,7 @@ import org.mule.api.transaction.TransactionConfig;
 import org.mule.api.transport.Connector;
 import org.mule.config.MuleManifest;
 import org.mule.config.i18n.CoreMessages;
+import org.mule.transport.polling.MessageProcessorPollingMessageReceiver;
 
 import java.beans.ExceptionListener;
 import java.util.List;
@@ -98,6 +99,11 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
                 ((Startable) getMessageProcessorChain(flowConstruct)).start();
             }
             getConnector().registerListener(this, getMessageProcessorChain(flowConstruct), flowConstruct);
+            MessageProcessor polledMp = getPolledMessageProcessor();
+            if (polledMp instanceof Startable)
+            {
+                 ((Startable)polledMp).start();
+            }
         }
         catch (Exception e)
         {
@@ -114,10 +120,15 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
             {
                 ((Stoppable) getMessageProcessorChain(flowConstruct)).stop();
             }
+            MessageProcessor polledMp = getPolledMessageProcessor();
+            if (polledMp instanceof Stoppable)
+            {
+                ((Stoppable)polledMp).stop();
+            }
         }
         catch (Exception e)
         {
-            throw new LifecycleException(CoreMessages.failedToStartInboundEndpoint(this), e, this);
+            throw new LifecycleException(CoreMessages.failedToStopInboundEndpoint(this), e, this);
         }
     }
 
@@ -139,9 +150,27 @@ public class DefaultInboundEndpoint extends AbstractEndpoint implements InboundE
         {
             ((Initialisable) processorChain).initialise();
         }
+        MessageProcessor polledMp = getPolledMessageProcessor();
+        if (polledMp instanceof MuleContextAware)
+        {
+            ((MuleContextAware) polledMp).setMuleContext(getMuleContext());
+        }
+        if (polledMp instanceof FlowConstructAware)
+        {
+            ((FlowConstructAware) polledMp).setFlowConstruct(flowContruct);
+        }
+        if (polledMp instanceof Initialisable)
+        {
+            ((Initialisable) polledMp).initialise();
+        }
         return processorChain;
     }
 
+    protected MessageProcessor getPolledMessageProcessor()
+    {
+        return (MessageProcessor) getProperty(MessageProcessorPollingMessageReceiver.SOURCE_MESSAGE_PROCESSOR_PROPERTY_NAME);
+    }
+
     public void setFlowConstruct(FlowConstruct flowConstruct)
     {
         this.flowConstruct = flowConstruct;
