@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultExpressionManager.java 19500 2010-09-09 17:29:17Z dzapata $
+ * $Id: DefaultExpressionManager.java 20441 2010-12-02 16:15:09Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -12,6 +12,7 @@ package org.mule.expression;
 import org.mule.api.MuleContext;
 import org.mule.api.MuleMessage;
 import org.mule.api.context.MuleContextAware;
+import org.mule.api.expression.ExpressionEnricher;
 import org.mule.api.expression.ExpressionEvaluator;
 import org.mule.api.expression.ExpressionManager;
 import org.mule.api.expression.ExpressionRuntimeException;
@@ -27,6 +28,7 @@ import java.util.Iterator;
 import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentHashMap;
 import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentMap;
 import edu.emory.mathcs.backport.java.util.concurrent.atomic.AtomicBoolean;
+
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -48,6 +50,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
     private TemplateParser parser = TemplateParser.createMuleStyleParser();
 
     private ConcurrentMap evaluators = new ConcurrentHashMap(8);
+    private ConcurrentMap enrichers = new ConcurrentHashMap(8);
 
     private MuleContext muleContext;
 
@@ -72,6 +75,22 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         evaluators.put(evaluator.getName(), evaluator);
     }
 
+    public void registerEnricher(ExpressionEnricher enricher)
+    {
+        if (enricher == null)
+        {
+            throw new IllegalArgumentException(CoreMessages.objectIsNull("enricher").getMessage());
+        }
+
+        final String name = enricher.getName();
+        // TODO MULE-3809 Eliminate duplicate evaluators registration
+        if (logger.isDebugEnabled())
+        {
+            logger.debug("Enrichers already contain an object named '" + name + "'.  The previous object will be overwritten.");
+        }
+        enrichers.put(enricher.getName(), enricher);
+    }
+
     /**
      * Checks whether an evaluator is registered with the manager
      *
@@ -82,7 +101,18 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
     {
         return evaluators.containsKey(name);
     }
-
+    
+    /**
+     * Checks whether an enricher is registered with the manager
+     *
+     * @param name the name of the expression enricher
+     * @return true if the enricher is registered with the manager, false otherwise
+     */
+    public boolean isEnricherRegistered(String name)
+    {
+        return enrichers.containsKey(name);
+    }
+    
     /**
      * Removes the evaluator with the given name
      *
@@ -102,6 +132,26 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         }
         return evaluator;
     }
+    
+    /**
+     * Removes the evaluator with the given name
+     *
+     * @param name the name of the evaluator to remove
+     */
+    public ExpressionEnricher unregisterEnricher(String name)
+    {
+        if (name == null)
+        {
+            return null;
+        }
+
+        ExpressionEnricher enricher = (ExpressionEnricher) enrichers.remove(name);
+        if (enricher instanceof Disposable)
+        {
+            ((Disposable) enricher).dispose();
+        }
+        return enricher;
+    }
 
     /**
      * Evaluates the given expression.  The expression should be a single expression definition with or without
@@ -160,6 +210,45 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         return evaluate(expression, name, message, failIfNull);
     }
 
+
+    public void enrich(String expression, MuleMessage message, Object object)
+        throws ExpressionRuntimeException
+    {
+        String enricherName;
+
+        if (expression == null)
+        {
+            throw new IllegalArgumentException(CoreMessages.objectIsNull("expression").getMessage());
+        }
+        if (expression.startsWith(DEFAULT_EXPRESSION_PREFIX))
+        {
+            expression = expression.substring(2, expression.length() - 1);
+        }
+        int i = expression.indexOf(":");
+        if (i > -1)
+        {
+            enricherName = expression.substring(0, i);
+            expression = expression.substring(i + DEFAULT_EXPRESSION_POSTFIX.length());
+        }
+        else
+        {
+            enricherName = expression;
+            expression = null;
+        }
+        enrich(expression, enricherName, message, object);
+    }
+
+    public void enrich(String expression, String enricherName, MuleMessage message, Object object)
+    {
+        ExpressionEnricher enricher = (ExpressionEnricher) enrichers.get(enricherName);
+        if (enricher == null)
+        {
+            throw new IllegalArgumentException(CoreMessages.expressionEnricherNotRegistered(enricherName)
+                .getMessage());
+        }
+        enricher.enrich(expression, message, object);
+    }
+
     /**
      * Evaluates the given expression.  The expression should be a single expression definition with or without
      * enclosing braces. i.e. "mule:serviceName" and "#[mule:serviceName]" are both valid. For situations where
@@ -195,6 +284,85 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         return result;
     }
 
+    public boolean evaluateBoolean(String expression, String evaluator, MuleMessage message)
+        throws ExpressionRuntimeException
+    {
+        return evaluateBoolean(expression, evaluator, message, false, false);
+    }
+
+    public boolean evaluateBoolean(String expression, MuleMessage message) throws ExpressionRuntimeException
+    {
+        return evaluateBoolean(expression, message, false, false);
+    }
+
+    public boolean evaluateBoolean(String expression,
+                                   String evaluator,
+                                   MuleMessage message,
+                                   boolean nullReturnsTrue,
+                                   boolean nonBooleanReturnsTrue) throws ExpressionRuntimeException
+    {
+        try
+        {
+            return resolveBoolean(evaluate(expression, evaluator, message, false), nullReturnsTrue,
+                nonBooleanReturnsTrue, expression);
+        }
+        catch (RequiredValueException e)
+        {
+            return nullReturnsTrue;
+        }
+    }
+
+    public boolean evaluateBoolean(String expression,
+                                   MuleMessage message,
+                                   boolean nullReturnsTrue,
+                                   boolean nonBooleanReturnsTrue) throws ExpressionRuntimeException
+    {
+        try
+        {
+            return resolveBoolean(evaluate(expression, message, false), nullReturnsTrue,
+                nonBooleanReturnsTrue, expression);
+        }
+        catch (RequiredValueException e)
+        {
+            return nullReturnsTrue;
+        }
+    }
+
+    protected boolean resolveBoolean(Object result,
+                                     boolean nullReturnsTrue,
+                                     boolean nonBooleanReturnsTrue,
+                                     String expression)
+    {
+        if (result == null)
+        {
+            return nullReturnsTrue;
+        }
+        else if (result instanceof Boolean)
+        {
+            return (Boolean) result;
+        }
+        else if (result instanceof String)
+        {
+            if (result.toString().toLowerCase().equalsIgnoreCase("false"))
+            {
+                return false;
+            }
+            else if (result.toString().toLowerCase().equalsIgnoreCase("true"))
+            {
+                return true;
+            }
+            else
+            {
+                return nonBooleanReturnsTrue;
+            }
+        }
+        else
+        {
+            logger.warn("Expression: " + expression + ", returned an non-boolean result. Returning: "
+                        + nonBooleanReturnsTrue);
+            return nonBooleanReturnsTrue;
+        }
+    }
 
     /**
      * Evaluates expressions in a given string. This method will iterate through each expression and evaluate it. If
@@ -229,7 +397,15 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         {
             public Object match(String token)
             {
-                return evaluate(token, message, failIfNull);
+                Object result = evaluate(token, message, failIfNull);
+                if (result instanceof MuleMessage)
+                {
+                    return ((MuleMessage) result).getPayload();
+                }
+                else
+                {
+                    return result;
+                }
             }
         }, expression);
     }
@@ -249,6 +425,19 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         }
         evaluators.clear();
     }
+    
+    public void clearEnrichers()
+    {
+        for (Iterator iterator = enrichers.values().iterator(); iterator.hasNext();)
+        {
+            ExpressionEnricher enricher = (ExpressionEnricher) iterator.next();
+            if (enricher instanceof Disposable)
+            {
+                ((Disposable) enricher).dispose();
+            }
+        }
+        enrichers.clear();
+    }
 
     public boolean isExpression(String string)
     {
@@ -323,4 +512,5 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
             throw new InvalidExpressionException(expression, "Expression string is not an expression.  Use isExpression(String) to validate first");
         }
     }
+
 }
