@@ -1,5 +1,5 @@
 /*
- * $Id: DefaultEntryPointResolverSet.java 20075 2010-11-04 20:11:25Z dirk.olmes $
+ * $Id: DefaultEntryPointResolverSet.java 20773 2010-12-16 07:05:53Z dirk.olmes $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -15,12 +15,9 @@ import org.mule.api.model.EntryPointResolverSet;
 import org.mule.api.model.InvocationResult;
 import org.mule.util.CollectionUtils;
 
+import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.Set;
-import java.util.concurrent.CopyOnWriteArraySet;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 
 /**
  * Provides the default implementation of an {@link org.mule.api.model.EntryPointResolverSet}
@@ -31,39 +28,34 @@ import org.apache.commons.logging.LogFactory;
  */
 public class DefaultEntryPointResolverSet implements EntryPointResolverSet
 {
-    protected final Log logger = LogFactory.getLog(getClass());
-
-    private final Set<EntryPointResolver> entryPointResolvers = new LinkedHashSet<EntryPointResolver>(4);    
-    private final Set<String> exceptions = new CopyOnWriteArraySet<String>();
+    private final Set<EntryPointResolver> entryPointResolvers = new LinkedHashSet<EntryPointResolver>(4);
 
     public Object invoke(Object component, MuleEventContext context) throws Exception
     {
-        try
+        Set<String> exceptions = new HashSet<String>();
+
+        for (EntryPointResolver resolver : entryPointResolvers)
         {
-            for (EntryPointResolver resolver : entryPointResolvers)
+            InvocationResult result = resolver.invoke(component, context);
+            if (result.getState() == InvocationResult.State.SUCCESSFUL)
             {
-                InvocationResult result = resolver.invoke(component, context);
-                if (result.getState() == InvocationResult.State.SUCCESSFUL)
+                return result.getResult();
+            }
+            else
+            {
+                if (result.hasError())
                 {
-                    return result.getResult();
-                }
-                else
-                {
-                    if (result.hasError())
-                    {
-                        exceptions.add(result.getErrorMessage());
-                    }
+                    exceptions.add(result.getErrorMessage());
                 }
             }
-            throw new EntryPointNotFoundException(CollectionUtils.toString(exceptions, true));
         }
-        finally
-        {
-            exceptions.clear();
-        }
-
+        throw new EntryPointNotFoundException(CollectionUtils.toString(exceptions, true));
     }
 
+    /**
+     * @return the entry point resolves configured in this resolver set. Note that access to the
+     * set is not thread safe. Client code must take proper precautions to synchronize.
+     */
     public Set<EntryPointResolver> getEntryPointResolvers()
     {
         return entryPointResolvers;
@@ -71,8 +63,11 @@ public class DefaultEntryPointResolverSet implements EntryPointResolverSet
 
     public void setEntryPointResolvers(Set<EntryPointResolver> entryPointResolvers)
     {
-        this.entryPointResolvers.clear();
-        this.entryPointResolvers.addAll(entryPointResolvers);
+        synchronized (this.entryPointResolvers)
+        {
+            this.entryPointResolvers.clear();
+            this.entryPointResolvers.addAll(entryPointResolvers);
+        }
     }
 
     public void addEntryPointResolver(EntryPointResolver resolver)
@@ -85,6 +80,9 @@ public class DefaultEntryPointResolverSet implements EntryPointResolverSet
 
     public boolean removeEntryPointResolver(EntryPointResolver resolver)
     {
-        return this.entryPointResolvers.remove(resolver);
+        synchronized (entryPointResolvers)
+        {
+            return this.entryPointResolvers.remove(resolver);
+        }
     }
 }
