@@ -1,5 +1,5 @@
 /*
- * $Id: RemoteDispatcher.java 19368 2010-09-05 05:19:34Z mike.schilling $
+ * $Id: RemoteDispatcher.java 20385 2010-11-29 20:25:26Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -100,7 +100,7 @@ public class RemoteDispatcher implements Disposable
 
         EndpointBuilder endpointBuilder = endpointFactory.getEndpointBuilder(endpoint);
         endpointBuilder.setExchangePattern(MessageExchangePattern.REQUEST_RESPONSE);
-        syncServerEndpoint = muleContext.getRegistry().lookupEndpointFactory().getOutboundEndpoint(
+        syncServerEndpoint = muleContext.getEndpointFactory().getOutboundEndpoint(
             endpointBuilder);
 
         wireFormat = requestWireFormat();
