@@ -1,5 +1,5 @@
 /*
- * $Id: MuleMessageService.java 19710 2010-09-23 16:29:07Z tcarlson $
+ * $Id: MuleMessageService.java 19770 2010-09-29 18:17:54Z tcarlson $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -13,12 +13,11 @@ package org.mule.module.jbpm;
 import org.mule.MessageExchangePattern;
 import org.mule.api.MuleMessage;
 import org.mule.module.bpm.MessageService;
-import org.mule.transport.bpm.ProcessConnector;
 
 import java.util.Map;
 
 /**
- * Proxy for the message-generation service provided by Mule.  The real service gets injected here by the {@link ProcessConnector}.
+ * Proxy for the message-generation service provided by Mule.  The real service gets injected here by the {@link ProcessComponent}.
  */
 public class MuleMessageService implements MessageService
 {
