@@ -1,5 +1,5 @@
 /*
- * $Id: MuleResourceAdapter.java 19191 2010-08-25 21:05:23Z tcarlson $
+ * $Id: MuleResourceAdapter.java 20385 2010-11-29 20:25:26Z dfeist $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -307,7 +307,7 @@ public class MuleResourceAdapter implements ResourceAdapter, Serializable
 
         endpointBuilder.setExchangePattern(MessageExchangePattern.ONE_WAY);
 
-        return muleContext.getRegistry().lookupEndpointFactory().getInboundEndpoint(endpointBuilder);
+        return muleContext.getEndpointFactory().getInboundEndpoint(endpointBuilder);
     }
 
     /**
