@@ -1,5 +1,5 @@
 /*
- * $Id: JmsConnector.java 19191 2010-08-25 21:05:23Z tcarlson $
+ * $Id: JmsConnector.java 20501 2010-12-08 00:52:19Z tcarlson $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -32,6 +32,8 @@ import org.mule.transaction.TransactionCoordination;
 import org.mule.transport.AbstractConnector;
 import org.mule.transport.ConnectException;
 import org.mule.transport.jms.i18n.JmsMessages;
+import org.mule.transport.jms.redelivery.AutoDiscoveryRedeliveryHandlerFactory;
+import org.mule.transport.jms.redelivery.RedeliveryHandlerFactory;
 import org.mule.transport.jms.xa.ConnectionFactoryWrapper;
 import org.mule.util.BeanUtils;
 
