@@ -1,5 +1,5 @@
 /*
- * $Id: JettyNamespaceHandler.java 19191 2010-08-25 21:05:23Z tcarlson $
+ * $Id: JettyNamespaceHandler.java 20411 2010-11-30 21:33:29Z dandiep $
  * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
  *
@@ -10,6 +10,7 @@
 package org.mule.transport.servlet.jetty;
 
 import org.mule.config.spring.handlers.AbstractMuleNamespaceHandler;
+import org.mule.config.spring.parsers.generic.ChildDefinitionParser;
 
 /**
  * Registers a Bean Definition Parser for handling <code><jetty:connector></code> elements.
@@ -21,6 +22,7 @@ public class JettyNamespaceHandler extends AbstractMuleNamespaceHandler
     {
         registerMetaTransportEndpoints(JettyHttpConnector.JETTY);
         registerConnectorDefinitionParser(JettyHttpConnector.class);
+        registerBeanDefinitionParser("webapps", new ChildDefinitionParser("webappsConfiguration", WebappsConfiguration.class, true));
     }
 
 }
